import numpy as np


class TeachableMachineLite(object):
    '''
    Create your TeachableMachineLite object to run your exported AI models using TFLite.
    '''
    __supported_types = ('tflite', 'tensorflowlite', 'TensorFlowLite')

    def __init__(self, model_path='model.tflite', labels_file_path='labels.txt', model_type='tflite') -> None:
        self.model_type = model_type.lower()
        self.model_path = model_path
        self.labels_file_path = labels_file_path
        self.object_creation_status = self.model_type in self.__supported_types
        if self.object_creation_status:
            print('Teachable Machine Lite Object is created successfully.')
        else:
            raise RuntimeError(
                """*****Not supported model type: Select a supported model type like "tflite" or "tensorflowlite".*****""")

    def get_image_dimensions(self, interpreter):
        _, height, width, _ = interpreter.get_input_details()[0]['shape']
        return {
            'height': height,
            'width': width
        }

    def transform_image(self, interpreter, image):
        tensor_index = interpreter.get_input_details()[0]['index']
        input_tensor = interpreter.tensor(tensor_index)()[0]
        input_tensor[:, :] = image

    def classify_image(self, interpreter, top_k=1):
        output_details = interpreter.get_output_details()[0]
        output = np.squeeze(
            interpreter.get_tensor(output_details['index']))
        try:
            if output_details['dtype'] == np.uint8:
                scale, zero_point = output_details['quantization']
                output = scale * (output - zero_point)
        except:
            raise RuntimeError(
                """Not quantized model, there is no match with uint8 data type.""")
        ordered = np.argpartition(-output, top_k)
        predictions = [(i, output[i]) for i in ordered[:top_k]]
        highest_class_id, highest_class_prob = predictions[0]
        return {
            'highest_class_id': highest_class_id,
            'highest_class_prob': highest_class_prob
        }



from setuptools import setup, find_packages
from pathlib import Path

this_directory = Path(__file__).parent

VERSION = '0.0.1'
DESCRIPTION = 'A Python package to simplify the deployment process of exported Teachable Machine models into different embedded systems environments like Raspberry Pi and other SBCs using TensorFlowLite.'
LONG_DESCRIPTION = (this_directory / "README.md").read_text()

# Setting up
setup(
    name="teachable-machine-lite",
    version=VERSION,
    author="Meqdad Dev (Meqdad Darwish)",
    author_email="meqdad.darweesh@gmail.com",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=['numpy', 'tflite-runtime'],
    url='https://github.com/MeqdadDev/teachable-machine-lite',
    download_url='https://github.com/MeqdadDev/teachable-machine-lite',
    keywords=['python', 'teachable machine', 'ai', 'computer vision',
              'camera', 'opencv', 'image classification', 'tensorflowlite'],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "License :: OSI Approved :: MIT License"
    ]
)
